@@ -56,7 +56,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#if !defined(_WIN32) && !defined(_WIN64)
+#if !(defined(_WIN32) || defined(_WIN64) || defined(__SVR4) || defined(__svr4__))
 #   include <sys/cdefs.h>
 #   include <err.h>
 #else
