@@ -3,7 +3,7 @@
  * @ingroup MAT
  */
 /*
- * Copyright (c) 2015-2023, The matio contributors
+ * Copyright (c) 2015-2025, The matio contributors
  * Copyright (c) 2005-2014, Christopher C. Hulbert
  * All rights reserved.
  *
@@ -56,14 +56,16 @@
 mat_t *
 Mat_Create4(const char *matname)
 {
-    FILE *fp = NULL;
+    FILE *fp;
     mat_t *mat = NULL;
 
-#if defined(_WIN32) && defined(_MSC_VER)
+#if defined(_WIN32)
     wchar_t *wname = utf82u(matname);
     if ( NULL != wname ) {
         fp = _wfopen(wname, L"w+b");
         free(wname);
+    } else {
+        fp = NULL;
     }
 #else
     fp = fopen(matname, "w+b");
@@ -108,8 +110,9 @@ Mat_Create4(const char *matname)
  * @endif
  */
 int
-Mat_VarWrite4(mat_t *mat, matvar_t *matvar)
+Mat_VarWrite4(const mat_t *mat, const matvar_t *matvar)
 {
+#if defined(MATIO_LE) || defined(MATIO_BE)
     typedef struct
     {
         mat_int32_t type;
@@ -119,13 +122,17 @@ Mat_VarWrite4(mat_t *mat, matvar_t *matvar)
         mat_int32_t namelen;
     } Fmatrix;
 
-    mat_uint32_t i;
     Fmatrix x;
 
     if ( NULL == mat || NULL == matvar )
         return MATIO_E_BAD_ARGUMENT;
     if ( NULL == matvar->name || matvar->rank != 2 )
         return MATIO_E_OUTPUT_BAD_DATA;
+    if ( matvar->class_type == MAT_C_SPARSE ) {
+        const mat_sparse_t *sparse = (const mat_sparse_t *)matvar->data;
+        if ( NULL == matvar->data || sparse->njc == 0 )
+            return MATIO_E_OUTPUT_BAD_DATA;
+    }
 
     switch ( matvar->data_type ) {
         case MAT_T_DOUBLE:
@@ -150,26 +157,8 @@ Mat_VarWrite4(mat_t *mat, matvar_t *matvar)
             return MATIO_E_OUTPUT_BAD_DATA;
     }
 
-#if defined(__GLIBC__)
-#if ( __BYTE_ORDER == __LITTLE_ENDIAN )
-#elif (__BYTE_ORDER == __BIG_ENDIAN)
+#if defined(MATIO_BE)
     x.type += 1000;
-#else
-    return MATIO_E_OPERATION_NOT_SUPPORTED;
-#endif
-#elif defined(_BIG_ENDIAN) && !defined(_LITTLE_ENDIAN)
-    x.type += 1000;
-#elif defined(_LITTLE_ENDIAN) && !defined(_BIG_ENDIAN)
-#elif defined(__sparc) || defined(__sparc__) || defined(_POWER) || defined(__powerpc__) || \
-    defined(__ppc__) || defined(__hpux) || defined(_MIPSEB) || defined(_POWER) ||          \
-    defined(__s390__)
-    x.type += 1000;
-#elif defined(__i386__) || defined(__alpha__) || defined(__ia64) || defined(__ia64__) ||   \
-    defined(_M_IX86) || defined(_M_IA64) || defined(_M_ALPHA) || defined(__amd64) ||       \
-    defined(__amd64__) || defined(_M_AMD64) || defined(__x86_64) || defined(__x86_64__) || \
-    defined(_M_X64) || defined(__bfin__) || defined(__loongarch64)
-#else
-    return MATIO_E_OPERATION_NOT_SUPPORTED;
 #endif
 
     x.namelen = (mat_int32_t)strlen(matvar->name) + 1;
@@ -200,9 +189,7 @@ Mat_VarWrite4(mat_t *mat, matvar_t *matvar)
             fwrite(&x, sizeof(Fmatrix), 1, (FILE *)mat->fp);
             fwrite(matvar->name, sizeof(char), x.namelen, (FILE *)mat->fp);
             if ( matvar->isComplex ) {
-                mat_complex_split_t *complex_data;
-
-                complex_data = (mat_complex_split_t *)matvar->data;
+                const mat_complex_split_t *complex_data = (mat_complex_split_t *)matvar->data;
                 fwrite(complex_data->Re, matvar->data_size, nelems, (FILE *)mat->fp);
                 fwrite(complex_data->Im, matvar->data_size, nelems, (FILE *)mat->fp);
             } else {
@@ -213,7 +200,7 @@ Mat_VarWrite4(mat_t *mat, matvar_t *matvar)
         case MAT_C_SPARSE: {
             mat_sparse_t *sparse;
             double tmp;
-            mat_uint32_t j;
+            mat_uint32_t i, j;
             size_t stride = Mat_SizeOf(matvar->data_type);
 #if !defined(EXTENDED_SPARSE)
             if ( MAT_T_DOUBLE != matvar->data_type )
@@ -247,12 +234,9 @@ Mat_VarWrite4(mat_t *mat, matvar_t *matvar)
             fwrite(&tmp, sizeof(double), 1, (FILE *)mat->fp);
             tmp = 0.;
             if ( matvar->isComplex ) {
-                mat_complex_split_t *complex_data;
-                char *re, *im;
-
-                complex_data = (mat_complex_split_t *)sparse->data;
-                re = (char *)complex_data->Re;
-                im = (char *)complex_data->Im;
+                mat_complex_split_t *complex_data = (mat_complex_split_t *)sparse->data;
+                const char *re = (char *)complex_data->Re;
+                const char *im = (char *)complex_data->Im;
                 for ( i = 0; i < sparse->njc - 1; i++ ) {
                     for ( j = sparse->jc[i]; j < sparse->jc[i + 1] && j < sparse->ndata; j++ ) {
                         fwrite(re + j * stride, stride, 1, (FILE *)mat->fp);
@@ -265,7 +249,7 @@ Mat_VarWrite4(mat_t *mat, matvar_t *matvar)
                     }
                 }
             } else {
-                char *data = (char *)sparse->data;
+                const char *data = (char *)sparse->data;
                 for ( i = 0; i < sparse->njc - 1; i++ ) {
                     for ( j = sparse->jc[i]; j < sparse->jc[i + 1] && j < sparse->ndata; j++ ) {
                         fwrite(data + j * stride, stride, 1, (FILE *)mat->fp);
@@ -280,6 +264,9 @@ Mat_VarWrite4(mat_t *mat, matvar_t *matvar)
     }
 
     return MATIO_E_NO_ERROR;
+#else
+    return MATIO_E_OPERATION_NOT_SUPPORTED;
+#endif
 }
 
 /** @if mat_devman
@@ -317,14 +304,13 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
             if ( matvar->isComplex ) {
                 mat_complex_split_t *complex_data = ComplexMalloc(matvar->nbytes);
                 if ( NULL != complex_data ) {
-                    size_t readcount;
-                    readcount =
+                    err =
                         ReadDoubleData(mat, (double *)complex_data->Re, matvar->data_type, nelems);
-                    err = readcount != nelems;
-                    readcount =
-                        ReadDoubleData(mat, (double *)complex_data->Im, matvar->data_type, nelems);
-                    err |= readcount != nelems;
-                    if ( 0 == err ) {
+                    if ( !err ) {
+                        err = ReadDoubleData(mat, (double *)complex_data->Im, matvar->data_type,
+                                             nelems);
+                    }
+                    if ( !err ) {
                         matvar->data = complex_data;
                     } else {
                         ComplexFree(complex_data);
@@ -337,9 +323,8 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
             } else {
                 matvar->data = malloc(matvar->nbytes);
                 if ( NULL != matvar->data ) {
-                    const size_t readcount =
-                        ReadDoubleData(mat, (double *)matvar->data, matvar->data_type, nelems);
-                    if ( readcount != nelems ) {
+                    err = ReadDoubleData(mat, (double *)matvar->data, matvar->data_type, nelems);
+                    if ( err ) {
                         free(matvar->data);
                         matvar->data = NULL;
                         return MATIO_E_FILE_FORMAT_VIOLATION;
@@ -357,9 +342,8 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
             matvar->nbytes = nelems;
             matvar->data = malloc(matvar->nbytes);
             if ( NULL != matvar->data ) {
-                const size_t readcount =
-                    ReadUInt8Data(mat, (mat_uint8_t *)matvar->data, matvar->data_type, nelems);
-                if ( readcount != nelems ) {
+                err = ReadUInt8Data(mat, (mat_uint8_t *)matvar->data, matvar->data_type, nelems);
+                if ( err ) {
                     free(matvar->data);
                     matvar->data = NULL;
                     return MATIO_E_FILE_FORMAT_VIOLATION;
@@ -379,51 +363,53 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
                 mat_sparse_t *sparse;
                 mat_off_t fpos;
                 enum matio_types data_type = MAT_T_DOUBLE;
-                size_t readcount;
+                size_t nBytes;
 
                 /* matvar->dims[1] either is 3 for real or 4 for complex sparse */
                 matvar->isComplex = matvar->dims[1] == 4 ? 1 : 0;
-                if ( matvar->dims[0] < 2 ) {
+                if ( 0 == matvar->dims[0] ) {
                     return MATIO_E_FILE_FORMAT_VIOLATION;
                 }
                 sparse = (mat_sparse_t *)matvar->data;
                 sparse->nir = (mat_uint32_t)(matvar->dims[0] - 1);
                 sparse->nzmax = sparse->nir;
-                err = Mul(&readcount, sparse->nir, sizeof(mat_uint32_t));
-                if ( err ) {
-                    Mat_Critical("Integer multiplication overflow");
-                    return err;
-                }
-                sparse->ir = (mat_uint32_t *)malloc(readcount);
-                if ( sparse->ir != NULL ) {
-                    readcount = ReadUInt32Data(mat, sparse->ir, data_type, sparse->nir);
-                    if ( readcount != sparse->nir ) {
-                        free(sparse->ir);
-                        free(matvar->data);
-                        matvar->data = NULL;
-                        return MATIO_E_FILE_FORMAT_VIOLATION;
-                    }
-                    for ( i = 0; i < sparse->nir; i++ ) {
-                        if ( 0 == sparse->ir[i] ) {
-                            err = MATIO_E_FILE_FORMAT_VIOLATION;
-                            break;
-                        }
-                        sparse->ir[i] = sparse->ir[i] - 1;
-                    }
+                if ( sparse->nir > 0 ) {
+                    err = Mul(&nBytes, sparse->nir, sizeof(mat_uint32_t));
                     if ( err ) {
-                        free(sparse->ir);
-                        free(matvar->data);
-                        matvar->data = NULL;
+                        Mat_Critical("Integer multiplication overflow");
                         return err;
                     }
-                } else {
-                    free(matvar->data);
-                    matvar->data = NULL;
-                    Mat_Critical("Couldn't allocate memory for the sparse row array");
-                    return MATIO_E_OUT_OF_MEMORY;
+                    sparse->ir = (mat_uint32_t *)malloc(nBytes);
+                    if ( sparse->ir != NULL ) {
+                        err = ReadUInt32Data(mat, sparse->ir, data_type, sparse->nir);
+                        if ( err ) {
+                            free(sparse->ir);
+                            free(matvar->data);
+                            matvar->data = NULL;
+                            return MATIO_E_FILE_FORMAT_VIOLATION;
+                        }
+                        for ( i = 0; i < sparse->nir; i++ ) {
+                            if ( 0 == sparse->ir[i] ) {
+                                err = MATIO_E_FILE_FORMAT_VIOLATION;
+                                break;
+                            }
+                            sparse->ir[i] = sparse->ir[i] - 1;
+                        }
+                        if ( err ) {
+                            free(sparse->ir);
+                            free(matvar->data);
+                            matvar->data = NULL;
+                            return err;
+                        }
+                    } else {
+                        free(matvar->data);
+                        matvar->data = NULL;
+                        Mat_Critical("Couldn't allocate memory for the sparse row array");
+                        return MATIO_E_OUT_OF_MEMORY;
+                    }
                 }
-                readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                if ( readcount != 1 || tmp > UINT_MAX - 1 || tmp < 0 ) {
+                err = ReadDoubleData(mat, &tmp, data_type, 1);
+                if ( err || tmp > UINT_MAX - 1 || tmp < 0 ) {
                     free(sparse->ir);
                     free(matvar->data);
                     matvar->data = NULL;
@@ -441,8 +427,8 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
                     return MATIO_E_FILE_FORMAT_VIOLATION;
                 }
                 (void)fseeko((FILE *)mat->fp, sparse->nir * Mat_SizeOf(data_type), SEEK_CUR);
-                readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                if ( readcount != 1 || tmp > UINT_MAX - 1 || tmp < 0 ) {
+                err = ReadDoubleData(mat, &tmp, data_type, 1);
+                if ( err || tmp > UINT_MAX - 1 || tmp < 0 ) {
                     free(sparse->ir);
                     free(matvar->data);
                     matvar->data = NULL;
@@ -459,42 +445,50 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
                     return MATIO_E_FILE_FORMAT_VIOLATION;
                 }
                 sparse->njc = (mat_uint32_t)matvar->dims[1] + 1;
-                err = Mul(&readcount, sparse->njc, sizeof(mat_uint32_t));
+                err = Mul(&nBytes, sparse->njc, sizeof(mat_uint32_t));
                 if ( err ) {
                     Mat_Critical("Integer multiplication overflow");
                     return err;
                 }
-                sparse->jc = (mat_uint32_t *)malloc(readcount);
-                if ( sparse->jc != NULL ) {
-                    mat_uint32_t *jc;
-                    err = Mul(&readcount, sparse->nir, sizeof(mat_uint32_t));
-                    if ( err ) {
-                        Mat_Critical("Integer multiplication overflow");
-                        return err;
-                    }
-                    jc = (mat_uint32_t *)malloc(readcount);
-                    if ( jc != NULL ) {
-                        mat_uint32_t j = 0;
-                        sparse->jc[0] = 0;
-                        readcount = ReadUInt32Data(mat, jc, data_type, sparse->nir);
-                        if ( readcount != sparse->nir ) {
+                if ( sparse->nir > 0 ) {
+                    sparse->jc = (mat_uint32_t *)malloc(nBytes);
+                    if ( sparse->jc != NULL ) {
+                        mat_uint32_t *jc;
+                        err = Mul(&nBytes, sparse->nir, sizeof(mat_uint32_t));
+                        if ( err ) {
+                            Mat_Critical("Integer multiplication overflow");
+                            return err;
+                        }
+                        jc = (mat_uint32_t *)malloc(nBytes);
+                        if ( jc != NULL ) {
+                            mat_uint32_t j = 0;
+                            sparse->jc[0] = 0;
+                            err = ReadUInt32Data(mat, jc, data_type, sparse->nir);
+                            if ( err ) {
+                                free(jc);
+                                free(sparse->jc);
+                                free(sparse->ir);
+                                free(matvar->data);
+                                matvar->data = NULL;
+                                return MATIO_E_FILE_FORMAT_VIOLATION;
+                            }
+                            for ( i = 1; i < sparse->njc - 1; i++ ) {
+                                while ( j < sparse->nir && jc[j] <= i )
+                                    j++;
+                                sparse->jc[i] = j;
+                            }
                             free(jc);
+                            /* terminating nnz */
+                            sparse->jc[sparse->njc - 1] = sparse->nir;
+                        } else {
                             free(sparse->jc);
                             free(sparse->ir);
                             free(matvar->data);
                             matvar->data = NULL;
-                            return MATIO_E_FILE_FORMAT_VIOLATION;
+                            Mat_Critical("Couldn't allocate memory for the sparse index array");
+                            return MATIO_E_OUT_OF_MEMORY;
                         }
-                        for ( i = 1; i < sparse->njc - 1; i++ ) {
-                            while ( j < sparse->nir && jc[j] <= i )
-                                j++;
-                            sparse->jc[i] = j;
-                        }
-                        free(jc);
-                        /* terminating nnz */
-                        sparse->jc[sparse->njc - 1] = sparse->nir;
                     } else {
-                        free(sparse->jc);
                         free(sparse->ir);
                         free(matvar->data);
                         matvar->data = NULL;
@@ -502,14 +496,10 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
                         return MATIO_E_OUT_OF_MEMORY;
                     }
                 } else {
-                    free(sparse->ir);
-                    free(matvar->data);
-                    matvar->data = NULL;
-                    Mat_Critical("Couldn't allocate memory for the sparse index array");
-                    return MATIO_E_OUT_OF_MEMORY;
+                    sparse->jc = (mat_uint32_t *)calloc(nBytes, 1);
                 }
-                readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                if ( readcount != 1 ) {
+                err = ReadDoubleData(mat, &tmp, data_type, 1);
+                if ( err ) {
                     free(sparse->jc);
                     free(sparse->ir);
                     free(matvar->data);
@@ -526,85 +516,97 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
 #if defined(EXTENDED_SPARSE)
                         switch ( data_type ) {
                             case MAT_T_DOUBLE:
-                                readcount = ReadDoubleData(mat, (double *)complex_data->Re,
-                                                           data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                                err |= readcount != 1;
-                                readcount = ReadDoubleData(mat, (double *)complex_data->Im,
-                                                           data_type, sparse->ndata);
-                                err |= readcount != sparse->ndata;
-                                readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadDoubleData(mat, (double *)complex_data->Re, data_type,
+                                                     sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadDoubleData(mat, &tmp, data_type, 1);
+                                }
+                                if ( !err ) {
+                                    err = ReadDoubleData(mat, (double *)complex_data->Im, data_type,
+                                                         sparse->ndata);
+                                }
+                                if ( !err ) {
+                                    err = ReadDoubleData(mat, &tmp, data_type, 1);
+                                }
                                 break;
                             case MAT_T_SINGLE: {
                                 float tmp2;
-                                readcount = ReadSingleData(mat, (float *)complex_data->Re,
-                                                           data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadSingleData(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
-                                readcount = ReadSingleData(mat, (float *)complex_data->Im,
-                                                           data_type, sparse->ndata);
-                                err |= readcount != sparse->ndata;
-                                readcount = ReadSingleData(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadSingleData(mat, (float *)complex_data->Re, data_type,
+                                                     sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadSingleData(mat, &tmp2, data_type, 1);
+                                }
+                                if ( !err ) {
+                                    err = ReadSingleData(mat, (float *)complex_data->Im, data_type,
+                                                         sparse->ndata);
+                                }
+                                if ( !err ) {
+                                    err = ReadSingleData(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             case MAT_T_INT32: {
                                 mat_int32_t tmp2;
-                                readcount = ReadInt32Data(mat, (mat_int32_t *)complex_data->Re,
-                                                          data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadInt32Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
-                                readcount = ReadInt32Data(mat, (mat_int32_t *)complex_data->Im,
-                                                          data_type, sparse->ndata);
-                                err |= readcount != sparse->ndata;
-                                readcount = ReadInt32Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadInt32Data(mat, (mat_int32_t *)complex_data->Re, data_type,
+                                                    sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadInt32Data(mat, &tmp2, data_type, 1);
+                                }
+                                if ( !err ) {
+                                    err = ReadInt32Data(mat, (mat_int32_t *)complex_data->Im,
+                                                        data_type, sparse->ndata);
+                                }
+                                if ( !err ) {
+                                    err = ReadInt32Data(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             case MAT_T_INT16: {
                                 mat_int16_t tmp2;
-                                readcount = ReadInt16Data(mat, (mat_int16_t *)complex_data->Re,
-                                                          data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadInt16Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
-                                readcount = ReadInt16Data(mat, (mat_int16_t *)complex_data->Im,
-                                                          data_type, sparse->ndata);
-                                err |= readcount != sparse->ndata;
-                                readcount = ReadInt16Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadInt16Data(mat, (mat_int16_t *)complex_data->Re, data_type,
+                                                    sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadInt16Data(mat, &tmp2, data_type, 1);
+                                }
+                                if ( !err ) {
+                                    err = ReadInt16Data(mat, (mat_int16_t *)complex_data->Im,
+                                                        data_type, sparse->ndata);
+                                }
+                                if ( !err ) {
+                                    err = ReadInt16Data(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             case MAT_T_UINT16: {
                                 mat_uint16_t tmp2;
-                                readcount = ReadUInt16Data(mat, (mat_uint16_t *)complex_data->Re,
-                                                           data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadUInt16Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
-                                readcount = ReadUInt16Data(mat, (mat_uint16_t *)complex_data->Im,
-                                                           data_type, sparse->ndata);
-                                err |= readcount != sparse->ndata;
-                                readcount = ReadUInt16Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadUInt16Data(mat, (mat_uint16_t *)complex_data->Re,
+                                                     data_type, sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadUInt16Data(mat, &tmp2, data_type, 1);
+                                }
+                                if ( !err ) {
+                                    err = ReadUInt16Data(mat, (mat_uint16_t *)complex_data->Im,
+                                                         data_type, sparse->ndata);
+                                }
+                                if ( !err ) {
+                                    err = ReadUInt16Data(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             case MAT_T_UINT8: {
                                 mat_uint8_t tmp2;
-                                readcount = ReadUInt8Data(mat, (mat_uint8_t *)complex_data->Re,
-                                                          data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadUInt8Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
-                                readcount = ReadUInt8Data(mat, (mat_uint8_t *)complex_data->Im,
-                                                          data_type, sparse->ndata);
-                                err |= readcount != sparse->ndata;
-                                readcount = ReadUInt8Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadUInt8Data(mat, (mat_uint8_t *)complex_data->Re, data_type,
+                                                    sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadUInt8Data(mat, &tmp2, data_type, 1);
+                                }
+                                if ( !err ) {
+                                    err = ReadUInt8Data(mat, (mat_uint8_t *)complex_data->Im,
+                                                        data_type, sparse->ndata);
+                                }
+                                if ( !err ) {
+                                    err = ReadUInt8Data(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             default:
@@ -620,16 +622,18 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
                                 return MATIO_E_FILE_FORMAT_VIOLATION;
                         }
 #else
-                        readcount = ReadDoubleData(mat, (double *)complex_data->Re, data_type,
-                                                   sparse->ndata);
-                        err = readcount != sparse->ndata;
-                        readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                        err |= readcount != 1;
-                        readcount = ReadDoubleData(mat, (double *)complex_data->Im, data_type,
-                                                   sparse->ndata);
-                        err |= readcount != sparse->ndata;
-                        readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                        err |= readcount != 1;
+                        err = ReadDoubleData(mat, (double *)complex_data->Re, data_type,
+                                             sparse->ndata);
+                        if ( !err ) {
+                            err = ReadDoubleData(mat, &tmp, data_type, 1);
+                        }
+                        if ( !err ) {
+                            err = ReadDoubleData(mat, (double *)complex_data->Im, data_type,
+                                                 sparse->ndata);
+                        }
+                        if ( !err ) {
+                            err = ReadDoubleData(mat, &tmp, data_type, 1);
+                        }
 #endif
                         if ( err ) {
                             ComplexFree(complex_data);
@@ -647,61 +651,61 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
                         Mat_Critical("Couldn't allocate memory for the complex sparse data");
                         return MATIO_E_OUT_OF_MEMORY;
                     }
-                } else {
+                } else if ( sparse->ndata > 0 ) {
                     sparse->data = malloc(sparse->ndata * Mat_SizeOf(data_type));
                     if ( sparse->data != NULL ) {
 #if defined(EXTENDED_SPARSE)
                         switch ( data_type ) {
                             case MAT_T_DOUBLE:
-                                readcount = ReadDoubleData(mat, (double *)sparse->data, data_type,
-                                                           sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadDoubleData(mat, (double *)sparse->data, data_type,
+                                                     sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadDoubleData(mat, &tmp, data_type, 1);
+                                }
                                 break;
                             case MAT_T_SINGLE: {
                                 float tmp2;
-                                readcount = ReadSingleData(mat, (float *)sparse->data, data_type,
-                                                           sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadSingleData(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadSingleData(mat, (float *)sparse->data, data_type,
+                                                     sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadSingleData(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             case MAT_T_INT32: {
                                 mat_int32_t tmp2;
-                                readcount = ReadInt32Data(mat, (mat_int32_t *)sparse->data,
-                                                          data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadInt32Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadInt32Data(mat, (mat_int32_t *)sparse->data, data_type,
+                                                    sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadInt32Data(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             case MAT_T_INT16: {
                                 mat_int16_t tmp2;
-                                readcount = ReadInt16Data(mat, (mat_int16_t *)sparse->data,
-                                                          data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadInt16Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadInt16Data(mat, (mat_int16_t *)sparse->data, data_type,
+                                                    sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadInt16Data(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             case MAT_T_UINT16: {
                                 mat_uint16_t tmp2;
-                                readcount = ReadUInt16Data(mat, (mat_uint16_t *)sparse->data,
-                                                           data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadUInt16Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadUInt16Data(mat, (mat_uint16_t *)sparse->data, data_type,
+                                                     sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadUInt16Data(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             case MAT_T_UINT8: {
                                 mat_uint8_t tmp2;
-                                readcount = ReadUInt8Data(mat, (mat_uint8_t *)sparse->data,
-                                                          data_type, sparse->ndata);
-                                err = readcount != sparse->ndata;
-                                readcount = ReadUInt8Data(mat, &tmp2, data_type, 1);
-                                err |= readcount != 1;
+                                err = ReadUInt8Data(mat, (mat_uint8_t *)sparse->data, data_type,
+                                                    sparse->ndata);
+                                if ( !err ) {
+                                    err = ReadUInt8Data(mat, &tmp2, data_type, 1);
+                                }
                                 break;
                             }
                             default:
@@ -717,11 +721,10 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
                                 return MATIO_E_FILE_FORMAT_VIOLATION;
                         }
 #else
-                        readcount =
-                            ReadDoubleData(mat, (double *)sparse->data, data_type, sparse->ndata);
-                        err = readcount != sparse->ndata;
-                        readcount = ReadDoubleData(mat, &tmp, data_type, 1);
-                        err |= readcount != 1;
+                        err = ReadDoubleData(mat, (double *)sparse->data, data_type, sparse->ndata);
+                        if ( !err ) {
+                            err = ReadDoubleData(mat, &tmp, data_type, 1);
+                        }
 #endif
                         if ( err ) {
                             free(sparse->data);
@@ -739,6 +742,16 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
                         Mat_Critical("Couldn't allocate memory for the sparse data");
                         return MATIO_E_OUT_OF_MEMORY;
                     }
+                } else {
+                    err = ReadDoubleData(mat, &tmp, data_type, 1);
+                    if ( err ) {
+                        free(sparse->data);
+                        free(sparse->jc);
+                        free(sparse->ir);
+                        free(matvar->data);
+                        matvar->data = NULL;
+                        return MATIO_E_FILE_FORMAT_VIOLATION;
+                    }
                 }
                 break;
             } else {
@@ -768,7 +781,8 @@ Mat_VarRead4(mat_t *mat, matvar_t *matvar)
  * @endif
  */
 int
-Mat_VarReadData4(mat_t *mat, matvar_t *matvar, void *data, int *start, int *stride, int *edge)
+Mat_VarReadData4(mat_t *mat, const matvar_t *matvar, void *data, const int *start,
+                 const int *stride, const int *edge)
 {
     int err = MATIO_E_NO_ERROR;
 
